/*===========================================================================
|> License
|	The MIT License (MIT)
|
|	Copyright (c) 2013 Edoardo Alberto Dominici
|
|	Permission is hereby granted, free of charge, to any person obtaining a copy
|	of this software and associated documentation files (the "Software"), to deal
|	in the Software without restriction, including without limitation the rights
|	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|	copies of the Software, and to permit persons to whom the Software is
|	furnished to do so, subject to the following conditions:
|
|	The above copyright notice and this permission notice shall be included in
|	all copies or substantial portions of the Software.
|
|	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
|	THE SOFTWARE.
=============================================================================*/

#ifndef EFFECT_HPP_INCLUDED_
#define EFFECT_HPP_INCLUDED_

/*===========================================================================
| > Includes
=============================================================================*/
#include "../common.hpp"
#include <unordered_map>

namespace ste
{
	namespace graphics
	{
		class Effect
		{
		public :
			Effect();
			Effect(const Effect& _other) = delete;
			Effect& operator=(const Effect& _other) = delete;

			virtual ~Effect();

			bool compile_and_load_from_file(const std::wstring& _filename);
			
			bool load_from_file(const std::wstring& _filename);
			bool load_from_memory(char* _data,size_t _size);

			ID3DX11Effect* get_internal_fx()const 
			{ 
				return m_internal_fx; 
			};
			
			ID3DBlob* get_raw_data()const { return m_raw_data; }

		private :
			ID3DX11Effect* m_internal_fx;
			ID3DBlob* m_raw_data;
		};
	}
}

#endif // EFFECT_HPP_INCLUDED_