/*===========================================================================
|> License
|	The MIT License (MIT)
|
|	Copyright (c) 2013 Edoardo Alberto Dominici
|
|	Permission is hereby granted, free of charge, to any person obtaining a copy
|	of this software and associated documentation files (the "Software"), to deal
|	in the Software without restriction, including without limitation the rights
|	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|	copies of the Software, and to permit persons to whom the Software is
|	furnished to do so, subject to the following conditions:
|
|	The above copyright notice and this permission notice shall be included in
|	all copies or substantial portions of the Software.
|
|	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
|	THE SOFTWARE.
=============================================================================*/

#ifndef RENDERER_HPP_INCLUDED_
#define RENDERER_HPP_INCLUDED_

/*===========================================================================
| > Includes
=============================================================================*/
#include "../common.hpp"

namespace ste
{
	namespace graphics
	{
		class IRenderable;
		class Sprite;

		/*===========================================================================
		| > Forward declarations
		=============================================================================*/
		class Window;

		class Renderer
		{
		public : 
			/*===========================================================================*/
			/// \brief Creates a new Renderer instance, but does not initialize Direct3D
			/*===========================================================================*/
			Renderer();

			/*===========================================================================*/
			/// \brief Destroyes the renderer and releases all the Direct3D associated resources
			/*===========================================================================*/
			~Renderer();

			//! Attaches the window
			/*===========================================================================*/
			/// \brief Attaches the window and creates all the Direct3D associated resources
			/*===========================================================================*/
			bool attach_window(Window* _window);
			 
			/*===========================================================================*/
			/// \brief Detaches the current attached window and releases all the Direct3D resources
			/*===========================================================================*/
			void detach_current_window();

			/*===========================================================================*/
			/// \brief Clears the backbuffer
			/*===========================================================================*/
			void clear();

			/*===========================================================================*/
			/// \brief Retrieves the current attached window if any
			/*===========================================================================*/
			inline Window* get_attached_window()const { return m_attached_window; } 

			/*===========================================================================*/
			/// Rendering methods
			/*===========================================================================*/
			void render(Sprite& _to_render);

		private : 
			bool _resize_buffers();
	
			IDXGISwapChain* m_swap_chain;
			ID3D11RenderTargetView* m_render_target_view;
		//	ID3D11DepthStencilView* m_depth_stencil_view;
			///ID3D11Texture2D* m_depth_stencil_buffer;
			D3D11_VIEWPORT m_viewport;

			Window* m_attached_window;
		};
	}
}

#endif // RENDERER_HPP_INCLUDED_